library(shipunov)

fragments <- c("coi", "its", "rbcl", "trnl", "matk") # fragile
sequences <- read.table("../../data/plantago_dna.txt", sep="\t", h=T, as.is=T, quote="")
treesp <- read.table("../../data/plantago_treesp.txt", sep="\t", h=T, as.is=T)
outgroups <- treesp$SPECIES.NEW[treesp$TYPE == "outgroup"]

dna <- structure(vector("list", 5), names=fragments) # fragile
for (f in fragments) {
dna[[f]] <- Read.fasta(paste0("33_alignments_trimmed_gapcoded/", f, ".fasta"))
sp.id <- do.call(rbind, strsplit(dna[[f]]$name, "__"))
dna[[f]] <- data.frame(SP=sp.id[, 1], ID=sp.id[, 2], SEQ=dna[[f]][, 2], stringsAsFactors=F)
dna[[f]] <- dna[[f]][order(dna[[f]]$SP, nchar(gsub("-", "", dna[[f]]$SEQ))), ]
}

## join all sp/ids from list
sp.ids <- do.call(rbind, c(lapply(dna, "[", 1:2), make.row.names=F))
tmp1 <- suppressWarnings(do.call(rbind, strsplit(sp.ids$ID, ", ")))
tmp2 <- unique(sort(paste(c(tmp1), sp.ids$SP, sep="__")))
ids.sp <- do.call(rbind, strsplit(tmp2, "__"))

## ===
## ===
## ===

## STEP 1. STRICT: its and trnl merged on "our" ids

## rows
tmp1 <- sequences[sequences$SOURCE == "MISU", "SEQUENCE.ID"] # not in buxineae (more uniform ids there)
tmp2 <- suppressWarnings(do.call(rbind, strsplit(tmp1, ", ")))
ourids.all <- unique(sort(tmp2))
ourids <- ids.sp[ids.sp[, 1] %in% ourids.all, ]

## make
strict <- data.frame(ourids, stringsAsFactors=F)
names(strict) <- c("ID", "SP")

## outgroups first, other sorted
tmp1 <- strict[strict$SP %in% outgroups, ]
tmp2 <- strict[!strict$SP %in% outgroups, ]
tmp2 <- tmp2[order(tmp2$SP), ]
strict <- rbind(tmp1, tmp2)

for (n in fragments[c(2, 4)]) { # fragile
 strict[[n]] <- NA
 for (i in 1:nrow(strict)) {
  ii <- dna[[n]]$SEQ[grepl(strict$ID[i], dna[[n]]$ID)]
  strict[[n]][i] <- ifelse(length(ii) == 0, NA, ii)
 }
}

## ===
## ===
## ===

## STEP 2. SEMI-STRICT (the main matrix):
## (1) add coi, matk and rbcl;
## (2) add new species which were not in "our" ids;
## (3) fill species sequences into all NA gaps and keep that id

semistrict <- strict

## rows
ssp <- sort(unique(semistrict$SP))
allsp <- sort(unique(sp.ids$SP))
addsp <- allsp[!allsp %in% ssp]
semistrict[(nrow(semistrict) + 1) : (nrow(semistrict) + length(addsp)), ] <- NA
semistrict[nrow(semistrict) : (nrow(semistrict) - (length(addsp) - 1)), "SP"] <- addsp
semistrict[nrow(semistrict) : (nrow(semistrict) - (length(addsp) - 1)), "ID"] <- NA

## outgroups first, other sorted
tmp1 <- semistrict[semistrict$SP %in% outgroups, ]
tmp2 <- semistrict[!semistrict$SP %in% outgroups, ]
tmp2 <- tmp2[order(tmp2$SP), ]
semistrict <- rbind(tmp1, tmp2)

## columns
names.tmp3 <- c(paste0(fragments, ".ID"), fragments[c(1, 3, 5)]) # fragile
tmp3 <- structure(data.frame(matrix(NA, nrow=nrow(semistrict), ncol=length(names.tmp3))), names=names.tmp3)
semistrict <- cbind(semistrict, tmp3)
semistrict <- semistrict[, c("SP", "ID", paste0(fragments, ".ID"), fragments)]

## main
for(n in fragments) {
 d <- dna[[n]]
 f.id <- paste0(n, ".ID")
 for (i in 1:nrow(semistrict)) {
  if (is.na(semistrict[i, n])) {
  dd <- d$SEQ[grepl(semistrict$SP[i], d$SP)]
  ii <- d$ID[grepl(semistrict$SP[i], d$SP)]
  if (length(dd) > 0 & length(ii) > 0) {
   semistrict[i, n] <- dd[1]
   semistrict[i, f.id] <- ii[1]
   } else {
   semistrict[i, f.id] <- "-"
   }
 }}
}

## ===
## ===
## ===

## STEP 3. Write files
## (1) semistrict (main) => will be used in mm, kmer and partial trees
## (2) tall (its + trnl)
## (3) broad (coi + rbcl + matk + tall subset)
## (4) mm => in the separate file

## semistrict statistics
semistrict.m <- data.frame(semistrict[, 1:2], apply(semistrict[, fragments], 2, function(.y) ifelse(is.na(.y), 0, 1)))
semistrict.m$COUNT <- rowSums(semistrict.m[, fragments])
## what missed after concatenation
semistrict.m[order(semistrict.m$COUNT, decreasing=T), ]
## how many ids in each missing state
table(apply(semistrict[, fragments], 1, function(.y) sum(!is.na(.y))))
## how many ids missed in each fragment
apply(semistrict[, fragments], 2, function(.x) paste0(sum(is.na(.x)), "/", length(.x)))
## how many sequences our and total
all.ids <- unlist(semistrict[, c("ID", paste0(fragments, ".ID"))])
## our
sum(all.ids %in% ourids.all & !is.na(all.ids) & all.ids != "-")
## total
sum(!is.na(all.ids) & all.ids != "-")
## number of potentially informative characters and total number of characters per fragment
for (f in fragments) {
cat(f)
tmp <- semistrict[, f]
tmp <- tmp[!is.na(tmp)]
tmp.table <- do.call(rbind, strsplit(tmp, split=""))
tmp.table[tmp.table == "-" | tmp.table == "N"] <- NA
cat("\tall chars ", ncol(tmp.table))
tmp.itic <- apply(tmp.table, 2, Is.tax.inform.char)
cat("\tpotentially taxonomically informative chars ", sum(tmp.itic), "\n")
}

## write original semistrict matrix
write.table(semistrict, file=paste0("40_concatenated/", "semistrict_orig.txt"), sep="\t", row.names=F, quote=F)

## write all duplicates, no gaps and N's FASTA (=> kmer)
seq.tmp <- apply(semistrict[, fragments], 2, function(.x) gsub("-", "", .x))
seq.tmp <- apply(seq.tmp, 2, function(.x) ifelse(is.na(.x), "", .x))
seq.tmp <- apply(seq.tmp, 1, function(.x) paste(.x, collapse=""))
## make joint ids
kmer.tmp <- semistrict
kmer.tmp.f.ids <- paste0(fragments, ".ID")
ids.tmp1 <- apply(kmer.tmp[, c("SP", "ID")], 1, function(.x) paste(.x, collapse="__"))
for (i in kmer.tmp.f.ids) kmer.tmp[, i] <- ifelse(is.na(kmer.tmp[, i]), "", paste(sub(".ID", "", i), kmer.tmp[, i]))
ids.tmp2 <- apply(kmer.tmp[, kmer.tmp.f.ids], 1, function(.x) paste(.x, collapse=" "))
ids.tmp2 <- gsub("(^ +)|( +$)", "", ids.tmp2)
ids.tmp2 <- gsub("( +)", " ", ids.tmp2)
ids.tmp <- paste(ids.tmp1, ids.tmp2)
ids.tmp <- gsub(" +", "_", ids.tmp)
ids.tmp <- gsub(",", "", ids.tmp)
ids.tmp <- gsub("_$", "", ids.tmp)
ids.tmp <- gsub("_NA_", "_", ids.tmp)
##
kmer <- data.frame(ID=ids.tmp, SEQ=seq.tmp, stringsAsFactors=F)
Write.fasta(kmer, file=paste0("40_concatenated/", "semistrict_kmer.fasta"))

## write semistrict fasta with joint ids, duplicates removed and NAs replaced with Ns
## duplicates
groups <- as.numeric(as.factor(apply(semistrict[, c("SP", fragments)], 1, paste, collapse="")))
comb.ids <-  aggregate(ids.tmp, by=list(groups), paste, collapse=" + ")
semistrict2 <- semistrict[!duplicated(groups), ] # remove duplicates in main table
ids <- ids.tmp[!duplicated(groups)] # remove duplicates in joint ids
comb.ids <- comb.ids[groups[!duplicated(groups)], 2] # the same order as in deduplicated data
## replace NA's with N's and write resulted FASTA
for (i in fragments) semistrict2[, i] <- ifelse(is.na(semistrict2[, i]), paste(rep("N", max(nchar(semistrict2[, i]), na.rm=T)), collapse=""), semistrict2[, i])
seq <- apply(semistrict2[, fragments], 1, function(.x) paste(.x, collapse=""))
semistrict.new <- data.frame(ID=ids, SEQ=seq, stringsAsFactors=FALSE)
Write.fasta(semistrict.new, file=paste0("40_concatenated/", "semistrict_orig.fasta"))

## ===
## make "tall" its + trnl
## (1) subset
## (2) make joint ids
## (3) remove duplicates and make combined ids
## (4) NA's into N's
## (5) write FASTA
## ===

tall.f <- c("its", "trnl")
tall.f.ids <- paste0(tall.f, ".ID")
tall <- semistrict[, c("SP", "ID", tall.f.ids, tall.f)]
tall <- tall[!rowSums(sapply(tall[, tall.f], is.na)) > 1, ] # remove rows where both markers missed

## make joint ids
ids.tmp0 <- tall[, c("SP", "ID", tall.f.ids)]
ids.tmp1 <- apply(ids.tmp0[, c("SP", "ID")], 1, function(.x) paste(.x, collapse="__"))
for (i in tall.f.ids) ids.tmp0[, i] <- ifelse(is.na(ids.tmp0[, i]), "", paste(sub(".ID", "", i), ids.tmp0[, i]))
ids.tmp2 <- apply(ids.tmp0[, tall.f.ids], 1, function(.x) paste(.x, collapse=" "))
ids.tmp2 <- gsub("(^ +)|( +$)", "", ids.tmp2)
ids.tmp2 <- gsub("( +)", " ", ids.tmp2)
ids.tmp <- paste(ids.tmp1, ids.tmp2)
ids.tmp <- gsub(" +", "_", ids.tmp)
ids.tmp <- gsub(",", "", ids.tmp)
ids.tmp <- gsub("_$", "", ids.tmp)
ids.tmp <- gsub("_NA_", "_", ids.tmp)

## duplicates
groups <- as.numeric(as.factor(apply(tall[, c("SP", tall.f)], 1, paste, collapse="")))
comb.ids <-  aggregate(ids.tmp, by=list(groups), paste, collapse=" + ")
tall <- tall[!duplicated(groups), ] # remove duplicates in main table
ids <- ids.tmp[!duplicated(groups)] # remove duplicates in joint ids
comb.ids <- comb.ids[groups[!duplicated(groups)], 2] # the same order as in deduplicated data
## this table is a "filter" for tree labels
write.table(data.frame(ids, comb.ids), file="40_concatenated/tall_comb_ids.txt", sep="\t", quote=F, row.names=F)

## tall statistics
tall.m <- data.frame(tall[, 1:2], apply(tall[, tall.f], 2, function(.y) ifelse(is.na(.y), 0, 1)))
tall.m$COUNT <- rowSums(tall.m[, tall.f])
## what missed after concatenation
tall.m[order(tall.m$COUNT, decreasing=T), ]
## how many ids in each missing state
table(apply(tall[, tall.f], 1, function(.y) sum(!is.na(.y))))
## how many ids missed in each fragment
apply(tall[, tall.f], 2, function(.x) paste0(sum(is.na(.x)), "/", length(.x)))
## how many sequences our and total
all.ids <- unlist(tall[, c("ID", paste0(tall.f, ".ID"))])
## our
sum(all.ids %in% ourids.all & !is.na(all.ids) & all.ids != "-")
## total
sum(!is.na(all.ids) & all.ids != "-")
## number of potentially informative characters and total number of characters per fragment
for (f in tall.f) {
cat(f)
tmp <- tall[, f]
tmp <- tmp[!is.na(tmp)]
tmp.table <- do.call(rbind, strsplit(tmp, split=""))
tmp.table[tmp.table == "-" | tmp.table == "N"] <- NA
cat("\tall chars ", ncol(tmp.table))
tmp.itic <- apply(tmp.table, 2, Is.tax.inform.char)
cat("\tpotentially taxonomically informative chars ", sum(tmp.itic), "\n")
}

## write tall matrix (=> mm)
write.table(tall, file=paste0("40_concatenated/", "tall.txt"), sep="\t", row.names=F, quote=F)

## replace NA's with N's and write resulted FASTA
for (i in tall.f) tall[, i] <- ifelse(is.na(tall[, i]), paste(rep("N", max(nchar(tall[, i]), na.rm=T)), collapse=""), tall[, i])
seq <- apply(tall[, tall.f], 1, function(.x) paste(.x, collapse=""))
tall.new <- data.frame(ID=ids, SEQ=seq, stringsAsFactors=F)
Write.fasta(tall.new, file=paste0("40_concatenated/", "tall.fasta"))

## ===
## make "broad"

broad.f <- fragments
broad.f.ids <- paste0(broad.f, ".ID")
broad <- semistrict[, c("SP", "ID", broad.f.ids, broad.f)]
broad <- broad[!rowSums(sapply(broad[, c("coi", "matk", "rbcl")], is.na)) == 3, ] # remove rows where broad markers missed

## make proper ids
ids.tmp0 <- broad[, c("SP", "ID", broad.f.ids)]
ids.tmp1 <- apply(ids.tmp0[, c("SP", "ID")], 1, function(.x) paste(.x, collapse="__"))
for (i in broad.f.ids) ids.tmp0[, i] <- ifelse(is.na(ids.tmp0[, i]), "", paste(sub(".ID", "", i), ids.tmp0[, i]))
ids.tmp2 <- apply(ids.tmp0[, broad.f.ids], 1, function(.x) paste(.x, collapse=" "))
ids.tmp2 <- gsub("(^ +)|( +$)", "", ids.tmp2)
ids.tmp2 <- gsub("( +)", " ", ids.tmp2)
ids.tmp <- paste(ids.tmp1, ids.tmp2)
ids.tmp <- gsub(" +", "_", ids.tmp)
ids.tmp <- gsub(",", "", ids.tmp)
ids.tmp <- gsub("_$", "", ids.tmp)
ids.tmp <- gsub("_NA_", "_", ids.tmp)

## duplicates
groups <- as.numeric(as.factor(apply(broad[, c("SP", broad.f)], 1, paste, collapse="")))
comb.ids <-  aggregate(ids.tmp, by=list(groups), paste, collapse=" + ")
broad <- broad[!duplicated(groups), ] # remove duplicates in main table
ids <- ids.tmp[!duplicated(groups)] # remove duplicates in joint ids
comb.ids <- comb.ids[groups[!duplicated(groups)], 2] # the same order as in deduplicated data
## this table is a "filter" for tree labels
write.table(data.frame(ids, comb.ids), file="40_concatenated/broad_comb_ids.txt", sep="\t", quote=F, row.names=F)

## broad statistics
broad.m <- data.frame(broad[, 1:2], apply(broad[, broad.f], 2, function(.y) ifelse(is.na(.y), 0, 1)))
broad.m$COUNT <- rowSums(broad.m[, broad.f])
## what missed after concatenation
broad.m[order(broad.m$COUNT, decreasing=T), ]
## how many ids in each missing state
table(apply(broad[, broad.f], 1, function(.y) sum(!is.na(.y))))
## how many ids missed in each fragment
apply(broad[, broad.f], 2, function(.x) paste0(sum(is.na(.x)), "/", length(.x)))
## how many sequences our and total
all.ids <- unlist(broad[, c("ID", paste0(broad.f, ".ID"))])
## our
sum(all.ids %in% ourids.all & !is.na(all.ids) & all.ids != "-")
## total
sum(!is.na(all.ids) & all.ids != "-")
## number of potentially informative characters and total number of characters per fragment
for (f in broad.f) {
cat(f)
tmp <- broad[, f]
tmp <- tmp[!is.na(tmp)]
tmp.table <- do.call(rbind, strsplit(tmp, split=""))
tmp.table[tmp.table == "-" | tmp.table == "N"] <- NA
cat("\tall chars ", ncol(tmp.table))
tmp.itic <- apply(tmp.table, 2, Is.tax.inform.char)
cat("\tpotentially taxonomically informative chars ", sum(tmp.itic), "\n")
}

## write broad matrix
write.table(broad, file=paste0("40_concatenated/", "broad.txt"), sep="\t", row.names=F, quote=F)

## replace NA's with N's
for (i in broad.f) broad[, i] <- ifelse(is.na(broad[, i]), paste(rep("N", max(nchar(broad[, i]), na.rm=T)), collapse=""), broad[, i])
seq <- apply(broad[, broad.f], 1, function(.x) paste(.x, collapse=""))
ids.new <- substr(ids, 0, 99) # dirty hack for MrBayes
write.table(data.frame(ids, ids.new), file="40_concatenated/broad_ids_mb.txt", sep="\t", quote=F, row.names=F) # to keep old ids
broad.new <- data.frame(ID=ids.new, SEQ=seq, stringsAsFactors=F)
Write.fasta(broad.new, file=paste0("40_concatenated/", "broad.fasta"))
